@@ -23,7 +23,6 @@ from ansiblelint._internal.rules import (
     RuntimeErrorRule,
     WarningRule,
 )
-from ansiblelint.app import App, get_app
 from ansiblelint.config import PROFILES, Options
 from ansiblelint.config import options as default_options
 from ansiblelint.constants import RULE_DOC_URL, SKIPPED_RULES_KEY
@@ -33,6 +32,7 @@ from ansiblelint.file_utils import Lintable, expand_paths_vars
 if TYPE_CHECKING:
     from ruamel.yaml.comments import CommentedMap, CommentedSeq
 
+    from ansiblelint.app import App
     from ansiblelint.errors import RuleMatchTransformMeta
 
 _logger = logging.getLogger(__name__)
@@ -390,12 +390,12 @@ class RulesCollection:
 
     def __init__(  # pylint: disable=too-many-arguments
         self,
+        app: App,
         rulesdirs: list[str] | list[Path] | None = None,
         options: Options | None = None,
         profile_name: str | None = None,
         *,
         conditional: bool = True,
-        app: App | None = None,
     ) -> None:
         """Initialize a RulesCollection instance."""
         if options is None:
@@ -406,10 +406,7 @@ class RulesCollection:
         else:
             self.options = options
         self.profile = []
-        # app should be defined on normal run logic, but for testing we might
-        # not pass it, and in this case we assume offline mode for performance
-        # reasons.
-        self.app = app or get_app(offline=True)
+        self.app = app
 
         if profile_name:
             self.profile = PROFILES[profile_name]
@@ -435,7 +432,9 @@ class RulesCollection:
         # When we have a profile we unload some of the rules
         # But we do include all rules when listing all rules or tags
         if profile_name and not (self.options.list_rules or self.options.list_tags):
-            filter_rules_with_profile(self.rules, profile_name)
+            filter_rules_with_profile(
+                self.rules, profile_name, self.options.enable_list
+            )
 
     def register(self, obj: AnsibleLintRule, *, conditional: bool = False) -> None:
         """Register a rule."""
@@ -507,27 +506,48 @@ class RulesCollection:
         for rule in self.rules:
             if rule.id == "syntax-check":
                 continue
+
+            is_targeted = any(t.startswith(f"{rule.id}[") for t in tags)
+
+            # rule selection logic
             if (
                 not tags
                 or rule.has_dynamic_tags
                 or not set(rule.tags).union([rule.id]).isdisjoint(tags)
+                or is_targeted
             ):
-                if tags and set(rule.tags).union(list(rule.ids().keys())).isdisjoint(
-                    tags,
+                # specific tag targeting override
+                if (
+                    tags
+                    and not is_targeted
+                    and set(rule.tags).union(list(rule.ids().keys())).isdisjoint(tags)
                 ):
-                    _logger.debug("Skipping rule %s", rule.id)
-                else:
-                    _logger.debug("Running rule %s", rule.id)
-                    rule_definition = set(rule.tags)
-                    rule_definition.add(rule.id)
-                    if set(rule_definition).isdisjoint(skip_list):
-                        matches.extend(rule.getmatches(file))
-            else:
-                _logger.debug("Skipping rule %s", rule.id)
+                    continue
 
-        # some rules can produce matches with tags that are inside our
-        # skip_list, so we need to cleanse the matches
-        matches = [m for m in matches if m.tag not in skip_list]
+                # rule-level skip check
+                rule_definition = set(rule.tags) | {rule.id}
+                if rule_definition.isdisjoint(skip_list):
+                    matches.extend(rule.getmatches(file))
+
+        if tags or skip_list:
+            filtered_matches = []
+            for m in matches:
+                # inclusion logic (if tags are provided)
+                if tags:
+                    if (
+                        m.tag in tags
+                        or m.rule.id in tags
+                        or (
+                            not set(m.rule.tags).isdisjoint(tags)
+                            and not any(t.startswith(f"{m.rule.id}[") for t in tags)
+                        )
+                    ):
+                        filtered_matches.append(m)
+                else:
+                    # no tags requested, so keep everything that wasn't skipped
+                    if m.tag not in skip_list:
+                        filtered_matches.append(m)
+            matches = filtered_matches
 
         return matches
 
@@ -579,9 +599,14 @@ class RulesCollection:
         return result
 
 
-def filter_rules_with_profile(rule_col: list[BaseRule], profile: str) -> None:
+def filter_rules_with_profile(
+    rule_col: list[BaseRule],
+    profile: str,
+    enable_list: list[str] | None = None,
+) -> None:
     """Unload rules that are not part of the specified profile."""
     included = set()
+    enabled = set(enable_list or [])
     extends = profile
     total_rules = len(rule_col)
     while extends:
@@ -592,7 +617,7 @@ def filter_rules_with_profile(rule_col: list[BaseRule], profile: str) -> None:
     for rule in rule_col.copy():
         if rule.unloadable:
             continue
-        if rule.id not in included:
+        if rule.id not in included and rule.id not in enabled:
             _logger.debug(
                 "Unloading %s rule due to not being part of %s profile.",
                 rule.id,
