@@ -2,6 +2,8 @@
 
 from __future__ import annotations
 
+import concurrent.futures
+import contextlib
 import json
 import logging
 import math
@@ -206,7 +208,7 @@ class Runner:
                 )
         return sorted(matches)
 
-    def _run(self) -> list[MatchError]:
+    def _run(self) -> list[MatchError]:  # noqa: C901
         """Run the linting (inner loop)."""
         files: list[Lintable] = []
         matches: list[MatchError] = []
@@ -297,12 +299,28 @@ class Runner:
 
             # avoid resource leak warning, https://github.com/python/cpython/issues/90549
             # pylint: disable=unused-variable
-            global_resource = multiprocessing.Semaphore()  # noqa: F841
+            with contextlib.suppress(OSError):
+                global_resource = multiprocessing.Semaphore()  # noqa: F841
 
-            pool = multiprocessing.pool.ThreadPool(processes=threads())
-            return_list = pool.map(worker, files, chunksize=1)
-            pool.close()
-            pool.join()
+            # In environments without /dev/shm (e.g., AWS Lambda/CodeBuild),
+            # multiprocessing.pool.ThreadPool fails because it still uses
+            # multiprocessing primitives (locks, queues). Fall back to
+            # concurrent.futures.ThreadPoolExecutor which is a pure threading
+            # implementation that doesn't require shared memory.
+            try:
+                pool = multiprocessing.pool.ThreadPool(processes=threads())
+                return_list = pool.map(worker, files, chunksize=1)
+                pool.close()
+                pool.join()
+            except (OSError, FileNotFoundError):
+                _logger.info(
+                    "ThreadPool creation failed (likely missing /dev/shm), "
+                    "falling back to concurrent.futures.ThreadPoolExecutor"
+                )
+                with concurrent.futures.ThreadPoolExecutor(
+                    max_workers=threads()
+                ) as executor:
+                    return_list = list(executor.map(worker, files))
             for data in return_list:
                 matches.extend(data)
 
